# Hello, this program is written in Python - http://python.org
'''
gossyp 0.1 DEV 2 - gossips from the internet

(see gossypDisplay_stdout.py for detailed documentation.)

For more information, see http://sebsauvage.net/python/gossyp/
'''
import os, random, urllib, re, sys, time, threading, Queue
import gossyppool

# Change your proxy here (if you use one):
#os.environ['http_proxy'] = 'http://127.0.0.1:3131'

debug = False

class fakePool:
    ''' If you do not have access to internet, you can use a text file
        (all_sentences.txt) containing sentences to fake fetchers.
    '''
    def __init__(self):
        print 'fakePool: Loading sentences...'
        file = open('all_sentences.txt','rb')
        s = file.read().split('\n')
        file.close()
        self._sentences = [i[:-1] for i in s if len(i.strip())>0]
        print 'fakePool: loaded %d messages.' % len(s)
    def getSentence(self):
        return random.choice(self._sentences)

class gossypDisplay_html:
    ''' Displays gossyp messages in a single HTML page. '''
    # HTML generation parameters:
    (_bg_r,_bg_g,_bg_b) = (0,54,85)      # Background color
    (_txt_r,_txt_g,_txt_b) = (0,155,174) # Foreground (text) color
    (_fontsize_min, _fontsize_max) = (12,35)  # Minimal and maximal fontsize (in HTML 'px' units)

    def __init__(self, filename='gossyp.html', refresh=None):
        ''' Creates the HTML page 'filename'.
            if refresh (integer) is specified, the page will be regenerated every n seconds
            (and the HTML document will contain a META REFRESH tag so that the
            browser will also reload the page every n seconds.)
        '''
        self._gp = gossyppool.gossypPool()
        #self._gp = fakePool()    # Use this pool instead of the real pool if you do not have internet access.
        print 'gossyp - HTML version'
        if refresh:
            print 'Generating a new page every %d seconds.' % refresh
            while True:
                self._makePage(filename,refresh)
                time.sleep(refresh)
        else:
            self._makePage(filename)

    def _makeRGB(self,red,green,blue):
        ''' Constructs and RGB triplet.
            Input: red,green,blue: integers between 0...255
            Output: Color as HTML code ('#a5c457')
        '''
        if red < 0  : red = 0
        if red > 255: red = 255
        if green < 0  : green = 0
        if green > 255: green = 255
        if blue < 0  : blue = 0
        if blue > 255: blue = 255
        return '#%02x%02x%02x' % (red,green,blue)

    def _makePage(self, filename, refresh=None):
        ''' Generate a page by getting sentences from the pool.
            if refresh is specified, the page will be generated
            with a META REFRESH tag.
        '''
        sentences = []
        while len(sentences)<10:
            sentence = self._gp.getSentence()
            if sentence:
                #file = open('sentences.txt','a')  # Save sentence to a file.
                #file.write(sentence+'\n\n')
                #file.close()
                sentences.append(sentence.strip())
            else:
                time.sleep(1)  # Wait a bit if the pool is empty.
        print 'gossypDisplay_html: Generating %s.' % filename
        html = self._makeHTML(sentences,refresh)
        file = open(filename,'w+b')
        file.write(html)
        file.close()

    def _makeDIV(self, sentence,fontsize,max):
        ''' Generate HTML DIV tag for a single sentense, using fontsize.
            fontsize if the font size in px unit.
            (max is maximum fontsize).
            Color is computed from fontsize.
            Position if randomly selected.
        '''
        _div = '<div class="a" style="left: %d%%; width: %d%%; top: %d%%; color: %s; font-size: %dpx">%s</div>\n'
        # Text color varies between foreground and background according to fontsize.
        # (the lowest the fontsize, the closer to the background color.)
        red   = self._bg_r+(self._txt_r-self._bg_r)*fontsize/max
        green = self._bg_g+(self._txt_g-self._bg_g)*fontsize/max
        blue  = self._bg_b+(self._txt_b-self._bg_b)*fontsize/max
        fontcolor = self._makeRGB(red,green,blue)
        left = random.randrange(70)
        width = random.randrange(20,70)
        if left+width>99: width = 99-left
        top = random.randrange(80)
        _div_shadow = '<div class="a" style="left: %d%%; width: %d%%; top: %d%%; color: %s; font-size: %dpx; padding-left: 1px; padding-top: 1px">%s</div>\n'
        html  = _div_shadow % (left,width,top,'#000000',fontsize,sentence)
        html += _div        % (left,width,top,fontcolor,fontsize,sentence)
        return html

    def _makeHTML(self, sentences,refresh=None):
        ''' Build the HTML file by generating the different parts:
            header, body...
            Input: sentences (list of string) contains the sentences to display in page.
                   refresh (integer), if specified, will add a REFESH meta tag in html header.
            Output: (a string) the generated html code.
        '''
        # Build the body
        body = ''
        lastsentence = sentences.pop()
        for i in range(len(sentences)):
            body += self._makeDIV(sentences[i], self._fontsize_min+(self._fontsize_max/1.5-self._fontsize_min)*i/len(sentences),self._fontsize_max)
            # The /1.5 is used to have all sentences fontsize not to close to the maximum fontsize.
        body += self._makeDIV(lastsentence,self._fontsize_max,self._fontsize_max)  # This last sentence will appear much bigger than others.

        # Build other tags (refresh, colors...)
        if refresh:
            refresh = '<meta http-equiv="refresh" content="%d">' % int(refresh)
        else:
            refresh = ''
        bgcolor = self._makeRGB(self._bg_r,self._bg_g,self._bg_b)
        biggossyp_color1 = self._makeRGB(self._bg_r+10,self._bg_g+10,self._bg_b+10)
        biggossyp_color2 = self._makeRGB(self._bg_r-10,self._bg_g-10,self._bg_b-10)
        smallgossyp_color = self._makeRGB(self._bg_r+120,self._bg_g+120,self._bg_b+120)

        # Build the whole HTML file:
        html ='''<html>
<head>
%s
<title>gossyp - gossips from the internet</title>
<style TYPE="text/css">
.a {  font-family: Arial, Helvetica, sans-serif; font-weight: bold; position: absolute; }
</style>
</head>
<body bgcolor="%s">
<div align="center" style="font-family: Arial, Helvetica, sans-serif; font-weight: bold; position: absolute; left: 0; width: 100%%; top: 25%%; color: %s; font-size: 200px; padding-left: 4px; padding-top: 4px">gossyp</div>
<div align="center" style="font-family: Arial, Helvetica, sans-serif; font-weight: bold; position: absolute; left: 0; width: 100%%; top: 25%%; color: %s; font-size: 200px">gossyp</div>
%s
<div align="right" style="font-family: Verdana, Arial, Helvetica, sans-serif; position: absolute; left: 1; width: 99%%; top: 1; color: #000000; font-size: 9px">page generated by <b>gossyp</b> - sebsauvage.net</div>
<div align="right" style="font-family: Verdana, Arial, Helvetica, sans-serif; position: absolute; left: 0; width: 99%%; top: 0; color: %s; font-size: 9px">page generated by <b>gossyp</b> - sebsauvage.net</div>
</body>
</html>
''' % (refresh,bgcolor,biggossyp_color1,biggossyp_color2,body,smallgossyp_color)
        return html



gossypDisplay_html('gossyp.html',30)
#gossypDisplay_html('gossyp.html')

