# Hello, this program is written in Python - http://python.org
'''
gossyp 0.1 DEV 2 - gossips from the internet

For more information, see http://sebsauvage.net/python/gossyp/

What ?
It's a weird, useless project of mine. gossyp fetches random gossips from the internet
(usenet, web, chat, forums...) and displays them on screen.


Why ?
Think of it as some kind of 'art'.
Or think of it as taking a snapshot of the current thoughts, rants and raves of people
all around the world.
You'll find grave truths, clever thoughts, stupid comments, insults, questions, quotes,
non-sense, deep sense, facts, out-of-the-context puzzling sentences from all over the world.

How ?
This program was written by Sebastien SAUVAGE - http://sebsauvage.net

How ?
gossyp fetches random messages from different sources (Usenet, web forums, IRC...),
extracts sentences, shuffles them and outputs them by various mean (standard output,
HTML page, speech, screensaver...)


Technical details:
There are 3 types of classes in gossyp:

- the gossypPool: This class is in charge of maitaining a pool of sentences.
  It also instanciates and starts all gossypFetcher_xxx classes.
  When the pool is running low on sentences, it asks idle gossypFetcher_xxx classes
  to fetch more sentences.
  The gossypPool will ask all fetchers to work until the pool reaches 2000 sentences.

- the gossypFetcher_xxx classes are in charge of fetching messages from various sources.
  For example, gossypFetcher_googleGroups fetches random sentences from recent Usenet messages
  using the Google Groups search engine.
  Each gossypFetcher_xxx works in its own thread so that other threads (and main thread)
  are not slowed down.

- the gossypDisplay_xxx classes are in charge of displaying sentences.
  They pick sentences one by one from gossypPool , and display them by various means.
  (For example, gossypDisplay_stdout displays sentences on standard output every 8 seconds.)

The whole thing uses a minuscule amout of CPU, a fair amount of RAM (< 6 Mo) and a good
amout of bandwith (especially when gossyp is starting.)

To do:
- write more gossypFetcher_xxx classes (especially to fetch messages from online chat,
  but that's a lot more work (we have to login, etc.))
- write more gossypDisplay_xxx classes (tk output, pyGame for nice output, screensaver,
  or even *speak* those sentences...)
- Improve filtering to get rid of non-sentences and broken sentences.
- better filtering of strange characters.
- individual gossypFetcher_xxx configuration (eg. fetch only some newsgroups or languagues in
  gossypFetcher_googleGroups, fetchs sentences between x and y characters only, etc.)
- add semantic concepts (fetch all messages within a semantic field, jump from homonyms to homonyms
  in different semantic fields, etc.)
- do not get the same sentences twice (hash tables).
- persistence of the pool to reduce warming up time and bandwith.
  (date each sentence ?)
- I should write a utility module, because some functions will probably be
  reused in different modules (message->sentences cut, rfc822->text, html->text functions, etc.)
- Be kind and parse robot rules.


WARNING:
    This program is provided as is. There is no garantee. Use it at your own risks.

License:
    Although this is public domain, please do not mirror this program.
    Please always put a link to the original page (http://sebsauvage.net/python/gossyp/)

Version:
    0.1 DEV 0:
        - First quick & dirty, mono-threaded, non-pooling version.
    0.1 DEV 1:
        - pooling
        - multithreading at least !   Much better.
    0.1 DEV 2:
        - split each module in a different py file.
        - created fecther module for Yahoo! Message Boards
          (http://messages.yahoo.com). Quite more complex than
          the Google Groups module, but still does not require login
          to read groups.
        - pool was extended from 1000 to 2000 sentences.
        - better error handling, but still greatly needs to be improved.
'''
import os, random, urllib, re, sys, time, threading, Queue
import gossyppool

# Change your proxy here (if you use one):
#os.environ['http_proxy'] = 'http://127.0.0.1:3131'

debug = False  # Set to true to see the pool working.

class fakePool:
    ''' If you do not have access to internet, you can use a text file
        (all_sentences.txt) containing sentences to fake fetchers.
    '''
    def __init__(self):
        print 'fakePool: Loading sentences...'
        file = open('all_sentences.txt','rb')
        s = file.read().split('\n')
        file.close()
        self._sentences = [i[:-1] for i in s if len(i.strip())>0]
        print 'fakePool: loaded %d messages.' % len(s)
    def getSentence(self):
        return random.choice(self._sentences)


class gossypDisplay_stdout:
    ''' Displays gossyp messages on standard output (console) every 8 seconds.
        Interrupt with CTRL+BREAK. '''
    def __init__(self):
        self._gp = gossyppool.gossypPool()
        #self._gp = fakePool()    # Use this pool instead of the real pool if you do not have internet access.
        sentence = gp.getSentence()
        if sentence is None: print 'gossyp warming up... please wait.\n'
        while True:
            sentence = gp.getSentence()
            if sentence is not None:
                print sentence+'\n'
                #file = open('sentences.txt','a')  # Save sentence to a file.
                #file.write(sentence+'\n\n')
                #file.close()
                time.sleep(8)
            else:
                time.sleep(1) # If the pool is empty, wait 1 second before querying the pool again.

gossypDisplay_stdout()  # Display messages on standard output.
