# Hello, this program is written in Python - http://python.org
import os, random, urllib, re, sys, time, threading, Queue
import token
from gossypfetcher_googlegroups import *
from gossypfetcher_yahoomb import *

# For more information, see http://sebsauvage.net/python/gossyp/

debug = False  # Set to true to see the pool working.

class gossypPool:
    ''' This class is in charge of instanciating and starting each gossypFetcher_xxx object,
        of maintaining a pool of sentences, and ask the gossypFetcher_xxx objects to fetch more
        sentences if the pool is going low. 
        Example: gp = gossypPool()
                 while True:
                     print gp.getSentence()
                     time.sleep(5)
        Note: once instanciated, you should call getSentence() at least once to force the
              fetcher to start working.
    '''
    def __init__(self):
        ''' Upon init, gossypPool instanciantes the differente fetcher threads and starts them.
            At this time, each thread does not start to fetch sentences. They're just waiting. '''
        if debug: print 'gossypPool: Creating and starting fetcher threads.'
        self._fetchers = []   # List of gossypFetcher_xxx objects
        self._sentencesQueue = Queue.Queue() # Sentences ouputed by the various the gossupFetcher_xxx classes.
        self._sentences = []  # Our pool of sentences.
        # All fetchers write to the same Queue (our self._sentencesQueue).
        # Each fetcher has its own command inQueue.

        # Create the fetchers
        self._fetchers.append( gossypFetcher_googleGroups(self._sentencesQueue) )
        self._fetchers.append( gossypFetcher_YahooMessageBoards(self._sentencesQueue) )

        # Start all fetcher threads.
        for fetcher in self._fetchers:
            fetcher.setDaemon(True)  # Python program will exit even if this threads is alive.
            fetcher.start()

    def poolFilter(self, sentence):
        ''' Pool-specific filter.
            This filter gets rids of sentence which are not desired.
            This filter is built from empirical tests.
            Input: a sentence (string).
            Output: the sentence (string), or None if the sentence must be dropped.
        '''
        if sentence.find('For a copy of the guidelines to this group, see')>=0: return None
        if sentence.find('All opinions expressed above are considered the opinions of the original poster')>=0: return None
        if sentence.find('Checked by NOD32')>=0: return None
        if sentence.lower().find('<br>')>=0: sentence = sentence.replace('<br>',' ')
        return sentence       

    def getSentence(self):
        ''' Returns a single sentence (and removes this sentence from the pool).
            If the pool is going low, this method will also ask inactive fetchers to
            fetch some more sentences.
            Output: a string.  None if no sentence is available. '''
        # Transfer sentences from the common output Queue to our private sentences pool.
        while not self._sentencesQueue.empty():
            try:
                sentence = self._sentencesQueue.get()
                if sentence is not None:
                    sentence = self.poolFilter(sentence)
                    if sentence: self._sentences.append(sentence)
            except Empty: pass

        # If the pool is running low, feed it:
        if len( self._sentences ) < 2000: # Keep a pool of a least 2000 sentences.
            # If the pool goes below, ask all idle fetchers to get some more sentences.
            for fetcher in self._fetchers:
                if fetcher.done: fetcher.inQueue.put(token.Token(fetch=1))

        # Return a random sentence from our pool of sentence, and remove it.
        if len(self._sentences)>0:
            i = random.randrange(len(self._sentences))
            if debug: print 'gossypPool: Sending sentence %d out of %d' % (i,len(self._sentences))
            return self._sentences.pop(i)
        else:
            return None

